/**
 * Inf+ module
 */
tma.registerModule( recfileinfplus );
function recfileinfplus() {

	/**
	 * IMPORTANT: the modules name; equal to directory and script file name
	 */
	this.name = 'recfileinfplus';

	var self = this;

	/**
	 * Tap ID
	 */
	this.TapId = 1145914122;

	/**
	 * override setup to check if Inf+ TAP is running
	 */
	this.setup = function() {

		// register tmaFeatures
		tma.addApiExtension( this, 'apiLoadInfPlus' );
		tma.fn.addFeature( this, 'showRecInf' );

		return $.Deferred().resolve();

	};

	/**
	 * get Inf+ info
	 *
	 * @param {String}		dir			path
	 * @param {TmaFile}		file		file object
	 * @param {String}		fileHex	hex representation of the filename
	 * @returns jqXHR
	 */
	this.apiLoadInfPlus = function (dir, file, fileHex) {

		var data =
			"dir="	    + dir + "\n" +
			"file="	    + file.name + "\n" +
			"file_hex=" + file.name_hex;

		var url = '/api?function=getlive&tap=' + this.TapId + '!';

		/**
		 * this construct allows us to switch to fail(),
		 * if the api returns an error_code <br>
		 * see: return deferred.promise().fail()
		 * http://stackoverflow.com/a/5112734
		 */
		var deferred = $.Deferred();

		$.ajax({
			type: "POST",
			url: url,
			data: data,
			dataType: 'json'
		}).done(function(data, status, xhr) {

			if (data.error_code === 0) {
				deferred.resolve( new InfPlus(data) );
			} else {
				// switch to fail()
				// TODO think about standardised errors
				deferred.reject(
						xhr,
						'tma-error',
						'loadInfPlus: POST ' + url + ' returns error_code ' + data.error_code
				);
			}

		}).fail(function(xhr, errorType, error) {
			deferred.reject(xhr, errorType, error);
		});

		return deferred.promise().fail(function(xhr, errorType, error) {

			tma.log({
				title: errorType + " in recfileinfplus.loadInfPlus()",
				details: {
					error: error,
					xhrReadyState: xhr.readyState,
					xhrStatus: xhr.status,
					parameter: '',
					directory: dir,
					file: file,
					filehex: fileHex,
				}
			});

		});

	};

	/**
	 * display Inf+ recording file info
	 *
	 * @param {tmaFeature}	f
	 * @param {TmaFile}		file		tma recfile object
	 */
	this.showRecInf = function(f, file) {

		// check, whether TAP is running
		if (!tma.taps.find(this.TapId)) {
			return false;
		}

		/**
		 * since this contains asynchronous calls, pause the chain
		 * and decide with the result of the call, whether to stop,
		 * or to continue
		 */
		f.pause();

		this.init().done(function() {

			var channel = tma.channel(file.infdata.header.logoid);

			var recStart = new Date(file.infdata.header.starttime);
			var recDuration = file.infdata.header.durationmin * 60000 + file.infdata.header.durationsec * 1000;
			var recEnd = new Date(recStart.getTime() + recDuration);

			var dateStr = (
					tma.date( tma.i18.eventstart_fmt, recStart ) + ' - ' +
					tma.date( 'H:i', recEnd) + ' &nbsp ' +
					tma.formatDuration(file.infdata.header.durationmin)
			);

			// test tma api extension
			tma.apiLoadInfPlus(tma.files.directory, file).done(
					/** @param {InfPlus} data */
					function(data) {

						data.setRelatedFile(file);

						var popup = new jkqWindow();
						popup.width = 500;

						popup.show();
						popup.setTitle(
								((channel) ? channel.servicename : file.infdata.service.servicename) + ' - ' + dateStr
						);
						popup.closeOnOutsideClick();

						var $content = popup.container.find('DIV.jkqWindowContent');

						var $infplusContent = $(
								self.templates.content.render({
									i18				: self.i18,
									events			: data.events,
									channel			: channel,
									fmtStart		: function() { return tma.date( tma.i18.eventstart_fmt, this.startDate() ); },
									fmtStartShort	: function() { return tma.date( tma.i18.eventstart_short, this.startDate() ); },
									fmtEnd			: function() { return tma.date( 'H:i', this.endDate() ); },
									fmtDuration		: function() { return tma.formatDuration(this.startEndDuration); },
								})
						).appendTo($content);

						$infplusContent.find('DIV.eventinfo[eventid="' + file.infdata.eventinfo.eventid + '"]')
						.show()
						.parents('DIV.fileinfo').css('background-color', '#FFFFFF');

						// accordion
						$infplusContent.on('click', 'DIV.fileinfo', function() {

							var $eventinfo = $(this).find('DIV.eventinfo');
							if ($eventinfo.is(':hidden')) {
								// open hidden infos only
								$infplusContent.find('DIV.eventinfo').filter(':visible')
								.slideUp()
								.parents('DIV.fileinfo').css('background-color', '#F0F0F0');
								$eventinfo.slideDown()
								.parents('DIV.fileinfo').css('background-color', '#FFFFFF');
							}

						});

						// play event
						$infplusContent.on('click', 'DIV.eventbar', function() {

							var startposition = $(this).attr('startblock');
							if (!startposition) startposition = 0;

							if (tma.fn.playFile) {
								tma.fn.playFile( tma.files.directory, file, startposition );
							}

						});

						popup.adjustPos({
							hAlign:	'right',
							effect:	'slide-rtl'
						});

						/**
						 * this method handled the feature, so
						 * stop the feature chain
						 * and return this popup instance
						 */
						f.stop(popup);

					}
			).fail(function(xhr, status, err) {

				/**
				 * this method can't handle the feature, so
				 * continue the feature chain
				 */
				f.run();

			});

		}).fail(function(xhr, status, err) {

			/**
			 * this method can't handle the feature, so
			 * continue the feature chain
			 */
			f.run();

		});

	};

}

function InfPlus( infplusdata ) {

	// empty default
	this.timestamp			= "";
	this.content			= "infplus";
	this.count				= 0;
	this.error_code			= 0;
	this.archived			= 0;
	this.requested_file		= '';
	this.requested_dir		= '';
	this.requested_file_hex	= '';
	this.requested_inode	= 0;
	this.events				= [];

	// apply initial data
	if (infplusdata) {

		if (infplusdata.events) {
			for( var i = 0; i < infplusdata.events.length; i++ ) {
				this.events.push( new InfPlusEvent() );
			}
		}

		tma.extend(this, infplusdata);

	}

	this.setRelatedFile = function(file) {

		var channel = tma.channel(file.infdata.header.logoid);

		var recStart = new Date(file.infdata.header.starttime);
		var recDuration = file.infdata.header.durationmin * 60000 + file.infdata.header.durationsec * 1000;
		var recEnd = new Date(recStart.getTime() + recDuration);
		var recBlocks = parseInt(file.size / tma.config.blockfactor);

		for( var i = 0; i < this.events.length; i++ ) {

			/** @returns {InfPlusEvent} */
			var event = this.events[i];

			event.startEndDuration	= parseInt( (event.endDate().getTime() - event.startDate().getTime()) / 60000);

			var infileStart			= Math.max(recStart.getTime(), event.startDate().getTime());
			var infileEnd			= Math.min(recEnd.getTime()  , event.endDate().getTime());
			event.infileDuration	= parseInt((infileEnd - infileStart) / 60000);

			event.isComplete		= (event.duration == event.infileDuration);

			event.startPercent		= parseInt( (infileStart - recStart)    * 1000 / recDuration ) / 10;
			event.durationPercent	= parseInt( (infileEnd   - infileStart) * 1000 / recDuration ) / 10;
			event.startBlock		= parseInt(recBlocks * event.startPercent/ 100);

		}

	};

}

function InfPlusEvent( event ) {

	var self = this;

	// empty default
	this.database_version	= 0;
	this.index				= 0;
	this.sequence			= 0;
	this.record_timestamp	= '';
	this.recordedsec		= 0;
	this.lcn				= 0;
	this.chname				= '';
	this.deleted			= 0;
	this.nextrecord			= 0;
	this.eventid			= 0;
	this.datastatus			= 0;
	this.runningstatus		= 0;
	this.starttime			= '';
	this.endtime			= '';
	this.timezone			= 0;
	this.duration			= 0;
	this.satindex			= 0;
	this.networkid			= 0;
	this.tsid				= 0;
	this.serviceid			= 0;
	this.servicetype		= 0;
	this.servicenum			= 0;
	this.lang				= '';
	this.parental			= 0;
	this.parental_text		= '';
	this.name				= '';
	this.description		= '';
	this.text				= '';
	this.citid				= 0;
	this.genre				= 0;
	this.genre_text			= '';
	this.sourceflag			= 0;
	this.icechannel			= 0;
	this.blocks				= 0;
	this.startblock			= 0;

	/**
	 * event start date
	 *
	 * @returns {Date}
	 */
	this.startDate = function() {
		return new Date(self.starttime);
	};

	/**
	 * event start date
	 *
	 * @returns {Date}
	 */
	this.endDate = function() {
		return new Date(self.endtime);
	};

	/**
	 * extended event text, where newlines are replaced by BR
	 *
	 * @returns {String}
	 */
	this.htmlText = function() {
		return self.text.replace(/(?:\r\n|\r|\n|\x8a)/g, '<br>');
	};

}
